from os import system
import time
from time import sleep
""" 
QUICKSTART
If you just want to run this without knowing why here is the short version.
However, I recommend reading the section labeled "IMPORTANT" below.

From the Chip Test Stand windows computer:
-open a Cygwin bash shell and from the 
home directory, enter: bin/backup.sh

On ohm.bu.edu:
While in the same directory that contains this script, run:
./copydatabase.sh
python VerifyChips.py

Now just follow the command prompt, while keeping the instructions
listed under "IMPORTANT" below on the expected order in which chips 
should be tested.




PURPOSE
This script is set up to verify the status of tested ASD chips for the ATLAS MDTs as
tested in the EDF at BU during a period of chip testing during 2012.
These were tested using a windows program developed at Harvard and a test stand developed 
at BU.  The results of the tests are saved to a database using Microsoft Access.  These
are then backed up to an EDF server called ohm.bu.edu under the account: bmc.
This Script Written by: Alex Long - Boston University - along528@bu.edu - 2012

BEFORE RUNNING THIS SCRIPT
A few preliminary steps are necessary before this script is ready to be used.
First, since this looks at the backed up database on ohm.bu.edu, it is useful
to backup the database yourself if you want to verify the most recently tested chips.
This may be done in the chip test stand computer inside of a Cygwin bash shell
with the command:
bin/backup.sh

This will move the most recently backed up database files to the directory ~bmc/ASD_Tests
In order to see all of the chips that have been tested it is necessary to look in all of the
mdb files with "cumulative" in the name as well as the the file "ASD.mdb"

This program expects a csv file that has been generated from the mdb file.
It looks for this single csv file at the location: ASD_Tests/ASD_All.csv
where ASD_Tests is a directory located in the same place as this script.

mdb-export EXAMPLE_INPUT.mdb ASDChip > EXAMPLE_OUTPUT.csv

An automated script called copydatabase.sh has been created which takes all of the
relevant mdb files from ohm, and creates a single csv file as is expected by 
this script.  Make sure copydatabase.sh is run in ohm.bu.edu and in the same directory in which this script
is located.

USAGE
run using:
python VerifyChips.py

When running the script you will first be prompted about what tray you want to verify.  You should respond with
either 1,2,3,4, or 5.  The default is 1.
Next, it will ask you for a name of the session.  This will
be used to specify a directory where the information about passed, failed, missing, and duplicate
chips will be stored. The default is the current date and the tray tested.

From here forward it will prompt for a barcode to verify in the database.
When a barcode is scanned it will check that the tray # matches the one that you are testing.
There are four classifications for a barcode that is entered:
PASSED - The tray # in the database for this chip matches the one expected.
FAILED - The tray # in the database for this chip is different from the one expected.
MISSING- The scanned barcode is not in the database.
DUPLICATE - This barcode has already been scanned.

Each time a chip is scanned, information is logged to the screen as well as to the appropriate log file (ex: 'FAILED.log')
In addition:
If PASSED
	The chip info is logged, the tally of # of chips scanned is incremented by 1, the system waits for the next chip to be scanned
If FAILED or MISSING
	The chip info is logged, the system beeps TWICE, and nothing else can happen until the user acknowledges that the 
	chip needs to be replaced by typing in 'ok'.
	The chip tally is NOT incremented.
If DUPLICATE
	The chip info is logged, the system beeps ONCE, and the system waits for the next chip to be scanned.
	The chip tally is NOT incremented.

Additional valid commands when prompted for a bar code include:
	'new' - reset the chip tally
	'plus' - increment the chip tally by 1
	'minus' - decrement the chip tally by 1
	'exit' - leave the program cleanly
	'help' - print a list of commands

Any other commands will be ignored.

********IMPORTANT**************
EXPECTED FLOW FOR SCANNING CHIP BARCODES
Since this is the last stage for verifying that a chip is valid before possibly
being used inside the detector, it is important that this verification has a 100% success rate.
In order to better accomplish this, the script has been made to expect a certain behavior from the user
which it can keep track of. As such, it has been designed to be intentionally inflexible.

This program expects you to be scanning chips using a barcode scanner as input.  Further, it assumes
that you are scanning a tray of 66 chips that is arranged with chips in a 6x11 grid and is setup to track
your progress so that you can make sure each chip has been checked.  
(From here onward I will refer
to the direction along the tray that is 6 chips wide (the shorter direction) as a row, and the direction
that is 11 chips long (the longer direction) as a column.)
A tally will be kept of how many chips you have scanned.
The script assumes that you are scanning along a single row at a time, notifying you each time you reach the end of a row.
Once the program thinks you have reached the end of a tray, you will here a system beep.
*** After the end of a tray you must enter "new" to reset the tally and to verify more chips***
*** No chips are verified if you do not reset the tally!***
The tally may be incremented by 1 by entering into the command prompt either 'plus' or 'minus'.
************************************


"""
def parsecsv(filename):
	file =open(filename,'r');
	database = []
	for line in file:
		line=line.rstrip()
		database.append(line.split(','))
	return database
	
def find( item,db,column):
  for value in range(len(db)):
	if(item==db[value][column]):
		return value

def isfound(item,db):
  for value in range(len(db)):
	if(item==db[value]):
		return True
  return False

	
def log(chipnum,db,tbid,filename,basedir):
	file = open(basedir+filename+'.log','a')
	#put logging here
	#have filenames declared inside here???
	logstring = "Time "+date()+" BarCode: "+db[chipnum][tbid['BarCode']]+' Operator: '+db[chipnum][tbid['Operator']]+' Timestamp '+db[chipnum][tbid['Timestamp']]+' TesterId: '+db[chipnum][tbid['TesterID']]+' Quality: '+db[chipnum][tbid['Quality']]+' Tray: '+db[chipnum][tbid['Tray']]+"\n"
	print logstring
	file.write(logstring)
	file.close()
	print filename
	
def logmiss(chipid,basedir):
	file = open(basedir+'MISSING.log','a')
	logstring = "Time "+date()+" BarCode: "+chipid+"\n"
	print logstring
	file.write(logstring)
	file.close()
	print 'MISSING'


def beep():
	#different kinds of beeps?
	system("echo -e '\a'")


def scan():
	input = raw_input("Enter the ChipID ")
	input = '"' + input + '"'
	return input

def date():
	return time.asctime(time.localtime(time.time()))
def dohelp():
	helpdoc="""
	Allowed commands in addition to a barcode:
		'new' - reset the tray tally
		'plus' - increment the tray tally by 1
		'minus' - decrement the tray tally by 1
		'exit' - leave the program cleanly
		'help' - print this
	"""
	print helpdoc
def fail(chipnum,db,tableid,basedir):
	log(chipnum,db,tableid,'FAIL',basedir)
	BeepAndWaitForOK()

def miss(chipid,basedir):
	logmiss(chipid,basedir)
	BeepAndWaitForOK()

def BeepAndWaitForOK():
	while True:
		beep(); sleep(.1); beep();
		input = raw_input("Enter 'ok' to acknowledge that chip needs to be replaced by a valid chip: ")
		if(input=="ok"):
			 break


#BEGIN MAIN

#setup inpufilename
inputfilename = "ASD_Tests/ASD_All.csv"

#get traynumber to test from user input
defaulttraytestnum='1'
while True:
	traytestnum=""
	traytestnum=raw_input("Enter a Tray number to test (Default: "+ defaulttraytestnum +") ")
	if(traytestnum==""): 
		traytestnum=defaulttraytestnum
		break
	
	
	if(not (traytestnum=='1' or traytestnum=='2' or traytestnum=='3' or traytestnum=='4' or traytestnum=='5')):
		print "\tInvalid entry"
		print traytestnum
	else:
		break


#get a name for this session to be used as a unique directory name for saving log files 
tempdate=date()
defaultsessionname=tempdate.replace(' ','-')+"_Tray"+str(traytestnum)
sessionname = raw_input("Enter a name to identify this test session (Default: "+defaultsessionname+") ") 
if(sessionname==""): sessionname=defaultsessionname
basedir=sessionname.replace(' ','')+'/'  #need to remove possible spaces
command="if [ ! -d  "+basedir+" ];then mkdir "+basedir+"; fi;";
system(command);



#generate database
db = parsecsv(inputfilename)

#generate dictionary for accessing appropriate columns
#called by one of the identifiers below
#IdChip,BarCode,WaferLotID,Operator,Timestamp,TesterID,FirmwareID,SoftwareID,Kp,Kn,V0p,V0n,Ix,Kn/Kp,Itotal,QualityTimeStamp,Quality,Tray,Voff_spread,Voff_correction,IdMezzBoard,RejCode,SR1,SR2,SR3,SR4,SR5,SR6,SR7,SR8,SR9,SR10,SR11
tableid = {}
for col in range(len(db[0])):
	tempdict = { db[0][col] : col }
	tableid.update(tempdict)
	tempdict.clear()


chipidscannedlist = []

traysize=66;
traytalley=0;

while True:
	#take chipid from scanner
	chipid = scan()
	#look for certain commands
	if(chipid=='"exit"' or chipid=='"Exit"' or chipid=='"EXIT"'):
		break
	if(chipid=='"help"' or chipid=='"Help"' or chipid=='"HELP"'):
		dohelp()
		continue
	elif(chipid=='"new"'):
		traytalley=0;
		print "Reset Traytalley to traytalley=",traytalley," of ",traysize
		continue
	elif(chipid=='"plus"'):
		traytalley+=1;
		print "Traytalley+1->",traytalley," of ",traysize
		continue
	elif(chipid=='"minus"'):
		traytalley-=1;
		print "Traytalley-1->",traytalley," of ",traysize
		continue
        elif( (chipid.find('-')!=3) or (chipid.rfind('-')!=7) ):
		print "Do not understand this format. Ignoring."
		continue
	#grab the chip location in the database from the barcode
	chipnum = find(chipid,db,tableid['BarCode'])
	if(chipnum!=None): #if the scan is not a duplicate and has a valid code then determine pass/fail
		#first check in memory to see that this chip
		#hasn't already been scanned during this session
		#if so then it should yell at you and wait for the next chip
		if(isfound(chipid,chipidscannedlist)):
			log(chipnum,db,tableid,'DUPLICATE',basedir)
			beep()
			continue 
		traynum=db[chipnum][tableid['Tray']]
		if(traynum==traytestnum):
			log(chipnum,db,tableid,'PASS',basedir)
			traytalley+=1; #iterate if PASS
		else: #if chip is not int  appropriate tray then yell and log it appropriately
			fail(chipnum,db,tableid,basedir)
			
	else: #if anything else happens then classify as missing and yell
		miss(chipid,basedir)

	#add scanned chip to list of already scanned chips
	chipidscannedlist.append(chipid)

	print traytalley," of ",traysize,"in tray scanned"
	if(traytalley%6==0):
		 print "End of Line"
	if(traytalley>=traysize):
		beep()
		print "Reached end of tray. Please type 'new' to proceed to next tray and reset talley."
			
